@@ -46,7 +46,8 @@ pub struct KGraphStat<F:Float> {
     mean_in_degree : usize,
     /// max incoming degree. Useful to choose between Compressed Storage Mat or dense Array
     max_in_degree : usize,
-    /// range statistics. We assume at this step we can use f32 
+    ///  We maintain quantiles on distances to first neighbours ad f32
+    /// This can serve as an indicator on relative density around a point.
     min_radius_q : CKMS<f32>,
 }  // end of KGraphStat
 
@@ -111,7 +112,7 @@ pub struct KGraph<F> {
     nbnodes: usize,
     /// neighbours\[i\] contains the indexes of neighbours node i sorted by increasing weight edge!
     /// all node indexing is done after indexation in node_set
-    pub neighbours : Vec<Vec<OutEdge<F>>>,
+    neighbours : Vec<Vec<OutEdge<F>>>,
     /// to keep track of current node indexes.
     node_set : IndexSet<DataId>,
 }   // end of struct KGraph
@@ -291,7 +292,7 @@ pub fn kgraph_from_hnsw_all<T, D, F>(hnsw : &Hnsw<T,D>, nbng : usize) -> std::re
         // keep only the asked size. Could we keep more ?
         if vec_tmp.len() < nbng {
             nb_point_below_nbng += 1;
-            log::warn!("neighbours must have {} neighbours, point {} got only {}", point_id, max_nbng, vec_tmp.len());
+            log::trace!("neighbours must have {} neighbours, point {} got only {}", max_nbng, point_id, vec_tmp.len());
             if vec_tmp.len() == 0 {
                 let p_id = point.get_point_id();
                 log::warn!(" graph will not be connected, isolated point at layer {}  , pos in layer {} ", p_id.0, p_id.1);
@@ -310,12 +311,15 @@ pub fn kgraph_from_hnsw_all<T, D, F>(hnsw : &Hnsw<T,D>, nbng : usize) -> std::re
     assert_eq!(neighbours.len(), nb_point);
     log::trace!("KGraph::exiting init_from_hnsw_all");
     // now we can fill some statistics on density and incoming degrees for nodes!
-    log::info!("mean number of neighbours obtained = {:.2e}", mean_nbng as f64 / nb_point as f64);
+    log::info!("mean number of neighbours obtained = {:.3e}", mean_nbng as f64 / nb_point as f64);
     log::info!("minimal number of neighbours {}", minimum_nbng);
     log::info!("number of points with less than : {} neighbours = {} ", nbng, nb_point_below_nbng);
-    if (mean_nbng as f64 / nb_point as f64) < nbng as f64 {
-        println!(" mean number of neighbours obtained : {:2.e}", mean_nbng);
-        println!(" possibly use hnsw.reset_keeping_pruned(true)");
+    let mean_nbng = mean_nbng as f64 / nb_point as f64;
+    if mean_nbng < nbng as f64 {
+        log::warn!(" mean number of neighbours obtained : {:.3e}", mean_nbng);
+        log::warn!(" possibly use hnsw.set_keeping_pruned(true)");
+        println!(" mean number of neighbours obtained : {:.3e}", mean_nbng);
+        println!(" possibly use hnsw.set_keeping_pruned(true)");
     }
     //
     Ok(KGraph{max_nbng, nbnodes, neighbours, node_set})
@@ -411,11 +415,12 @@ pub fn kgraph_from_hnsw_all<T, D, F>(hnsw : &Hnsw<T,D>, nbng : usize) -> std::re
         log::trace!("KGraph::exiting init_from_hnsw_layer");
         log::trace!("collected {} points", nb_point_collected);
         // now we can fill some statistics on density and incoming degrees for nodes!
-        log::info!("mean number of neighbours obtained = {:.2e}", mean_nbng as f64 / nb_point_collected as f64);
+        let mean_nbng = mean_nbng as f64 / nb_point_collected as f64;
+        log::info!("mean number of neighbours obtained = {:.3e}", mean_nbng);
         log::info!("minimal number of neighbours {}", minimum_nbng);
         log::info!("number of points with less than : {} neighbours = {} ", nbng, nb_point_below_nbng);
-        if (mean_nbng as f64 / nb_point as f64) < nbng as f64 {
-            println!(" mean number of neighbours obtained : {:2.e}", mean_nbng);
+        if mean_nbng < nbng as f64 {
+            println!(" mean number of neighbours obtained : {:.3e}", mean_nbng);
             println!(" possibly use hnsw.reset_keeping_pruned(true)");
         }
         //
@@ -607,6 +612,7 @@ impl <F> KGraphProjection<F>
         //
         log::info!("number of points with less than : {} neighbours = {} ", nbng, nb_point_below_nbng);
         let upper_graph = KGraph::<F>{ max_nbng : nbng, nbnodes : upper_graph_neighbours.len() , neighbours : upper_graph_neighbours, node_set : upper_index_set};
+        log::info!("getting stats from reduced graph");
         let _graph_stats = upper_graph.get_kraph_stats();
         //
         // construct the dense whole  graph, lower layers or more populated layers
@@ -655,7 +661,7 @@ impl <F> KGraphProjection<F>
         }  // end for on layers
         // we have both graph and projection
         let whole_graph = KGraph::<F>{ max_nbng : nbng, nbnodes : graph_neighbours.len() , neighbours : graph_neighbours, node_set : index_set};
-        log::trace!("getting stats from large graph");
+        log::info!("getting stats from whole graph");
         let _graph_stats = whole_graph.get_kraph_stats();
         // 
         log::info!("number of points with less than : {} neighbours = {} ", nbng, nb_point_below_nbng);
